@@ -1,7 +1,7 @@
 /*
  * wee-eval.c - evaluate expressions with references to internal vars
  *
- * Copyright (C) 2012-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2012-2022 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -181,7 +181,7 @@ eval_strstr_level (const char *string, const char *search,
 {
     const char *ptr_string;
     int level, length_search, debug_id;
-    int length_prefix, length_prefix2, length_suffix, length_suffix2;
+    int length_prefix2, length_suffix2;
 
     ptr_string = NULL;
 
@@ -196,9 +196,6 @@ eval_strstr_level (const char *string, const char *search,
 
     length_search = strlen (search);
 
-    length_prefix = strlen (eval_context->prefix);
-    length_suffix = strlen (eval_context->suffix);
-
     length_prefix2 = (extra_prefix) ? strlen (extra_prefix) : 0;
     length_suffix2 = (extra_suffix) ? strlen (extra_suffix) : 0;
 
@@ -213,10 +210,10 @@ eval_strstr_level (const char *string, const char *search,
         {
             ptr_string++;
         }
-        else if (strncmp (ptr_string, eval_context->prefix, length_prefix) == 0)
+        else if (strncmp (ptr_string, eval_context->prefix, eval_context->length_prefix) == 0)
         {
             level++;
-            ptr_string += length_prefix;
+            ptr_string += eval_context->length_prefix;
         }
         else if ((length_prefix2 > 0)
                  && (strncmp (ptr_string, extra_prefix, length_prefix2) == 0))
@@ -224,11 +221,11 @@ eval_strstr_level (const char *string, const char *search,
             level++;
             ptr_string += length_prefix2;
         }
-        else if (strncmp (ptr_string, eval_context->suffix, length_suffix) == 0)
+        else if (strncmp (ptr_string, eval_context->suffix, eval_context->length_suffix) == 0)
         {
             if (level > 0)
                 level--;
-            ptr_string += length_suffix;
+            ptr_string += eval_context->length_suffix;
         }
         else if ((length_suffix2 > 0)
                  && (strncmp (ptr_string, extra_suffix, length_suffix2) == 0))
@@ -276,6 +273,46 @@ eval_string_eval_cond (const char *text, struct t_eval_context *eval_context)
     return strdup ((rc) ? EVAL_STR_TRUE : EVAL_STR_FALSE);
 }
 
+/*
+ * Converts string to lower case.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_string_lower (const char *text)
+{
+    char *tmp;
+
+    tmp = strdup (text);
+    if (!tmp)
+        return strdup ("");
+
+    string_tolower (tmp);
+
+    return tmp;
+}
+
+/*
+ * Converts string to upper case.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_string_upper (const char *text)
+{
+    char *tmp;
+
+    tmp = strdup (text);
+    if (!tmp)
+        return strdup ("");
+
+    string_toupper (tmp);
+
+    return tmp;
+}
+
 /*
  * Hides chars in a string.
  *
@@ -346,7 +383,7 @@ eval_string_cut (const char *text, int screen)
     if (!pos2)
         return strdup ("");
 
-    tmp = strndup (text, pos - text);
+    tmp = string_strndup (text, pos - text);
     if (!tmp)
         return strdup ("");
 
@@ -358,7 +395,7 @@ eval_string_cut (const char *text, int screen)
     }
     free (tmp);
 
-    tmp = strndup (pos + 1, pos2 - pos - 1);
+    tmp = string_strndup (pos + 1, pos2 - pos - 1);
     if (!tmp)
         return strdup ("");
 
@@ -386,7 +423,7 @@ eval_string_repeat (const char *text)
     if (!pos)
         return strdup ("");
 
-    tmp = strndup (text, pos - text);
+    tmp = string_strndup (text, pos - text);
     if (!tmp)
         return strdup ("");
 
@@ -401,6 +438,273 @@ eval_string_repeat (const char *text)
     return string_repeat (pos + 1, number);
 }
 
+/*
+ * Splits string.
+ *
+ * Format: number,separators,flags,string
+ *
+ * If number == "count", returns the number of items after split.
+ * If number == "random", returns a random item.
+ * If number > 0, return this index (empty string if not enough items).
+ * If number < 0, return this index starting from the end (-1 = last item,
+ * -2 = penultimate item, etc.).
+ *
+ * If separators is empty string, a comma is used by default.
+ *
+ * Flags is a list of flags, separated by "+":
+ *   strip_left
+ *   strip_right
+ *   collapse_seps
+ *   keep_eol
+ *   strip_items=xyz
+ *   max_items=N
+ *
+ * Examples:
+ *   ${split:1,,,abc,def,ghi}                              ==> abc
+ *   ${split:-1,,,abc,def,ghi}                             ==> ghi
+ *   ${split:count,,,abc,def,ghi}                          ==> 3
+ *   ${split:random,,,abc,def,ghi}                         ==> def
+ *   ${split:3,,collapse_seps,abc,,,def,,,ghi}             ==> ghi
+ *   ${split:3,,strip_items=-_,_-abc-_,_-def-_,_-ghi-_}    ==> ghi
+ *   ${split:2, ,,this is a test}                          ==> is
+ *   ${split:2, ,strip_left+strip_right, this is a test }  ==> is
+ *   ${split:2, ,keep_eol,this is a test}                  ==> is a test
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_string_split (const char *text)
+{
+    char *pos, *pos2, *pos3, *str_number, *separators, **items, *value, *error;
+    char str_value[32], *str_flags, **list_flags, *strip_items;
+    int i, num_items, count_items, random_item, flags;
+    long number, max_items;
+
+    str_number = NULL;
+    separators = NULL;
+    items = NULL;
+    value = NULL;
+    str_flags = NULL;
+    list_flags = NULL;
+    strip_items = NULL;
+    count_items = 0;
+    random_item = 0;
+    flags = 0;
+    max_items = 0;
+
+    if (!text || !text[0])
+        goto end;
+
+    pos = strchr (text, ',');
+    if (!pos || (pos == text))
+        goto end;
+
+    number = 0;
+    str_number = string_strndup (text, pos - text);
+    if (strcmp (str_number, "count") == 0)
+    {
+        count_items = 1;
+    }
+    else if (strcmp (str_number, "random") == 0)
+    {
+        random_item = 1;
+    }
+    else
+    {
+        number = strtol (str_number, &error, 10);
+        if (!error || error[0] || (number == 0))
+            goto end;
+    }
+
+    pos++;
+    pos2 = strchr (pos, ',');
+    if (!pos2)
+        goto end;
+    if (pos2 == pos)
+        separators = strdup (",");
+    else
+        separators = string_strndup (pos, pos2 - pos);
+
+    pos2++;
+    pos3 = strchr (pos2, ',');
+    if (!pos3)
+        goto end;
+    str_flags = string_strndup (pos2, pos3 - pos2);
+    list_flags = string_split (str_flags, "+", NULL, 0, 0, NULL);
+    if (list_flags)
+    {
+        for (i = 0; list_flags[i]; i++)
+        {
+            if (strcmp (list_flags[i], "strip_left") == 0)
+                flags |= WEECHAT_STRING_SPLIT_STRIP_LEFT;
+            else if (strcmp (list_flags[i], "strip_right") == 0)
+                flags |= WEECHAT_STRING_SPLIT_STRIP_RIGHT;
+            else if (strcmp (list_flags[i], "collapse_seps") == 0)
+                flags |= WEECHAT_STRING_SPLIT_COLLAPSE_SEPS;
+            else if (strcmp (list_flags[i], "keep_eol") == 0)
+                flags |= WEECHAT_STRING_SPLIT_KEEP_EOL;
+            else if (strncmp (list_flags[i], "strip_items=", 12) == 0)
+            {
+                if (strip_items)
+                    free (strip_items);
+                strip_items = strdup (list_flags[i] + 12);
+            }
+            else if (strncmp (list_flags[i], "max_items=", 10) == 0)
+            {
+                max_items = strtol (list_flags[i] + 10, &error, 10);
+                if (!error || error[0] || (max_items < 0))
+                    goto end;
+            }
+        }
+    }
+
+    pos3++;
+
+    items = string_split (pos3, separators, strip_items, flags,
+                          max_items, &num_items);
+
+    /* if "count" was asked, return the number of items found after split */
+    if (count_items)
+    {
+        snprintf (str_value, sizeof (str_value), "%d", num_items);
+        value = strdup (str_value);
+        goto end;
+    }
+
+    if (!items || (num_items < 1))
+        goto end;
+
+    /* if "random" was asked, return a random item */
+    if (random_item)
+        number = random () % num_items;
+    else if (number > 0)
+        number--;
+
+    if (((number >= 0) && (number >= num_items))
+        || ((number < 0) && (labs (number) > num_items)))
+    {
+        goto end;
+    }
+
+    if (number < 0)
+        number = num_items + number;
+
+    value = strdup (items[number]);
+
+end:
+    if (str_number)
+        free (str_number);
+    if (separators)
+        free (separators);
+    if (str_flags)
+        free (str_flags);
+    if (list_flags)
+        string_free_split (list_flags);
+    if (strip_items)
+        free (strip_items);
+    if (items)
+        string_free_split (items);
+    return (value) ? value : strdup ("");
+}
+
+/*
+ * Splits shell arguments.
+ *
+ * Format: number,string
+ *
+ * If number == "count", returns the number of arguments.
+ * If number == "random", returns a random argument.
+ * If number > 0, return this index (empty string if not enough arguments).
+ * If number < 0, return this index starting from the end (-1 = last argument,
+ * -2 = penultimate argument, etc.).
+ *
+ * Examples:
+ *   ${split_shell:count,"arg 1" arg2}   ==> "2"
+ *   ${split_shell:random,"arg 1" arg2}  ==> "arg2"
+ *   ${split_shell:1,"arg 1" arg2}       ==> "arg 1"
+ *   ${split_shell:-1,"arg 1" arg2}      ==> "arg2"
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_string_split_shell (const char *text)
+{
+    char *pos, *str_number, **items, *value, *error, str_value[32];
+    int num_items, count_items, random_item;
+    long number;
+
+    str_number = NULL;
+    items = NULL;
+    value = NULL;
+    count_items = 0;
+    random_item = 0;
+
+    if (!text || !text[0])
+        goto end;
+
+    pos = strchr (text, ',');
+    if (!pos || (pos == text))
+        goto end;
+
+    number = 0;
+    str_number = string_strndup (text, pos - text);
+    if (strcmp (str_number, "count") == 0)
+    {
+        count_items = 1;
+    }
+    else if (strcmp (str_number, "random") == 0)
+    {
+        random_item = 1;
+    }
+    else
+    {
+        number = strtol (str_number, &error, 10);
+        if (!error || error[0] || (number == 0))
+            goto end;
+    }
+
+    pos++;
+
+    items = string_split_shell (pos, &num_items);
+
+    /* if "count" was asked, return the number of items found after split */
+    if (count_items)
+    {
+        snprintf (str_value, sizeof (str_value), "%d", num_items);
+        value = strdup (str_value);
+        goto end;
+    }
+
+    if (!items || (num_items < 1))
+        goto end;
+
+    /* if "random" was asked, return a random item */
+    if (random_item)
+        number = random () % num_items;
+    else if (number > 0)
+        number--;
+
+    if (((number >= 0) && (number >= num_items))
+        || ((number < 0) && (labs (number) > num_items)))
+    {
+        goto end;
+    }
+
+    if (number < 0)
+        number = num_items + number;
+
+    value = strdup (items[number]);
+
+end:
+    if (str_number)
+        free (str_number);
+    if (items)
+        string_free_split (items);
+    return (value) ? value : strdup ("");
+}
+
 /*
  * Returns a regex group captured.
  *
@@ -416,14 +720,24 @@ eval_string_regex_group (const char *text, struct t_eval_context *eval_context)
     if (!eval_context->regex || !eval_context->regex->result)
         return strdup ("");
 
-    if (strcmp (text, "+") == 0)
-        number = eval_context->regex->last_match;
-    else if (strcmp (text, "#") == 0)
+    if (strcmp (text, "#") == 0)
     {
         snprintf (str_value, sizeof (str_value),
                   "%d", eval_context->regex->last_match);
         return strdup (str_value);
     }
+
+    if (strcmp (text, "repl_index") == 0)
+    {
+        snprintf (str_value, sizeof (str_value),
+                  "%d", eval_context->regex_replacement_index);
+        return strdup (str_value);
+    }
+
+    if (strcmp (text, "+") == 0)
+    {
+        number = eval_context->regex->last_match;
+    }
     else
     {
         number = strtol (text, &error, 10);
@@ -676,7 +990,7 @@ eval_string_if (const char *text, struct t_eval_context *eval_context)
         (char *)eval_strstr_level (pos + 1, ":",
                                    eval_context, NULL, NULL, 1) : NULL;
     condition = (pos) ?
-        strndup (text, pos - text) : strdup (text);
+        string_strndup (text, pos - text) : strdup (text);
     if (!condition)
         return strdup ("");
     tmp = eval_expression_condition (condition, eval_context);
@@ -692,7 +1006,7 @@ eval_string_if (const char *text, struct t_eval_context *eval_context)
         if (pos)
         {
             tmp = (pos2) ?
-                strndup (pos + 1, pos2 - pos - 1) : strdup (pos + 1);
+                string_strndup (pos + 1, pos2 - pos - 1) : strdup (pos + 1);
             if (tmp)
             {
                 value = eval_replace_vars (tmp, eval_context);
@@ -725,6 +1039,57 @@ eval_string_if (const char *text, struct t_eval_context *eval_context)
     return (value) ? value : strdup ("");
 }
 
+/*
+ * Returns a random integer number.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+eval_string_random (const char *text)
+{
+    char *pos, *error, *tmp, result[128];
+    long long min_number, max_number;
+
+    if (!text || !text[0])
+        goto error;
+
+    pos = strchr (text, ',');
+    if (!pos)
+        goto error;
+
+    tmp = string_strndup (text, pos - text);
+    if (!tmp)
+        goto error;
+    min_number = strtoll (tmp, &error, 10);
+    if (!error || error[0])
+    {
+        free (tmp);
+        goto error;
+    }
+    free (tmp);
+
+    max_number = strtoll (pos + 1, &error, 10);
+    if (!error || error[0])
+        goto error;
+
+    if (min_number > max_number)
+        goto error;
+    if (max_number - min_number > RAND_MAX)
+        goto error;
+
+    /*
+     * using modulo division on the random() value produces a biased result,
+     * but this is enough for our usage here
+     */
+    snprintf (result, sizeof (result),
+              "%lld", min_number + (random () % (max_number - min_number + 1)));
+    return strdup (result);
+
+error:
+    return strdup ("0");
+}
+
 /*
  * Translates text.
  *
@@ -732,7 +1097,7 @@ eval_string_if (const char *text, struct t_eval_context *eval_context)
  */
 
 char *
-eval_translate (const char *text)
+eval_string_translate (const char *text)
 {
     const char *ptr_string;
 
@@ -744,6 +1109,28 @@ eval_translate (const char *text)
     return strdup ((ptr_string) ? ptr_string : "");
 }
 
+/*
+ * Defines a variable.
+ */
+
+void
+eval_string_define (const char *text, struct t_eval_context *eval_context)
+{
+    char *pos, *name;
+
+    pos = strchr (text, ',');
+    if (!pos)
+        return;
+
+    name = string_strndup (text, pos - text);
+    if (!name)
+        return;
+
+    hashtable_set (eval_context->user_vars, name, pos + 1);
+
+    free (name);
+}
+
 /*
  * Gets value of hdata using "path" to a variable.
  *
@@ -1028,36 +1415,46 @@ end:
 /*
  * Replaces variables, which can be, by order of priority:
  *   1. the string itself without evaluation (format: raw:xxx)
- *   2. an extra variable from hashtable "extra_vars"
+ *   2. a variable from hashtable "user_vars" or "extra_vars"
  *   3. a WeeChat home directory, one of: "weechat_config_dir",
  *      "weechat_data_dir", "weechat_cache_dir", "weechat_runtime_dir"
  *   4. a string to evaluate (format: eval:xxx)
  *   5. a condition to evaluate (format: eval_cond:xxx)
  *   6. a string with escaped chars (format: esc:xxx or \xxx)
- *   7. a string with chars to hide (format: hide:char,string)
- *   8. a string with max chars (format: cut:max,suffix,string or
+ *   7. a string converted to lower case (format: lower:xxx)
+ *   8. a string converted to upper case (format: upper:xxx)
+ *   9. a string with chars to hide (format: hide:char,string)
+ *  10. a string with max chars (format: cut:max,suffix,string or
  *      cut:+max,suffix,string) or max chars on screen
  *      (format: cutscr:max,suffix,string or cutscr:+max,suffix,string)
- *   9. a reversed string (format: rev:xxx) or reversed string for screen,
+ *  11. a reversed string (format: rev:xxx) or reversed string for screen,
  *      color codes are not reversed (format: revscr:xxx)
- *  10. a repeated string (format: repeat:count,string)
- *  11. length of a string (format: length:xxx) or length of a string on screen
+ *  12. a repeated string (format: repeat:count,string)
+ *  13. length of a string (format: length:xxx) or length of a string on screen
  *      (format: lengthscr:xxx); color codes are ignored
- *  12. a regex group captured (format: re:N (0.99) or re:+)
- *  13. a color (format: color:xxx)
- *  14. a modifier (format: modifier:name,data,xxx)
- *  15. an info (format: info:name,arguments)
- *  16. a base 16/32/64 encoded/decoded string (format: base_encode:base,xxx
+ *  14. split string (format: split:number,separators,flags,xxx
+ *      or split:count,separators,flags,xxx
+ *      or split:random,separators,flags,xxx)
+ *  15. split shell arguments (format: split:number,xxx or split:count,xxx
+ *      or split:random,xxx)
+ *  16. a regex group captured (format: re:N (0.99) or re:+)
+ *  17. a color (format: color:xxx)
+ *  18. a modifier (format: modifier:name,data,xxx)
+ *  19. an info (format: info:name,arguments)
+ *  20. a base 16/32/64 encoded/decoded string (format: base_encode:base,xxx
  *      or base_decode:base,xxx)
- *  17. current date/time (format: date or date:xxx)
- *  18. an environment variable (format: env:XXX)
- *  19. a ternary operator (format: if:condition?value_if_true:value_if_false)
- *  20. calculate result of an expression (format: calc:xxx)
- *  21. a translated string (format: translate:xxx)
- *  22. an option (format: file.section.option)
- *  23. a buffer local variable
- *  24. a pointer name from hashtable "pointers"
- *  25. a hdata variable (format: hdata.var1.var2 or hdata[list].var1.var2
+ *  21. current date/time (format: date or date:xxx)
+ *  22. an environment variable (format: env:XXX)
+ *  23. a ternary operator (format: if:condition?value_if_true:value_if_false)
+ *  24. calculate result of an expression (format: calc:xxx)
+ *  25. a random integer number in the range from "min" to "max"
+ *      (format: random:min,max)
+ *  26. a translated string (format: translate:xxx)
+ *  27. define a new variable (format: define:name,value)
+ *  28. an option (format: file.section.option)
+ *  29. a buffer local variable
+ *  30. a pointer name from hashtable "pointers"
+ *  31. a hdata variable (format: hdata.var1.var2 or hdata[list].var1.var2
  *                        or hdata[ptr].var1.var2 or hdata[ptr_name].var1.var2)
  *
  * See /help in WeeChat for examples.
@@ -1090,7 +1487,13 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 2. variable in hashtable "extra_vars" */
+    /* 2. variable in hashtable "user_vars" or "extra_vars" */
+    ptr_value = hashtable_get (eval_context->user_vars, text);
+    if (ptr_value)
+    {
+        value = strdup (ptr_value);
+        goto end;
+    }
     if (eval_context->extra_vars)
     {
         ptr_value = hashtable_get (eval_context->extra_vars, text);
@@ -1169,7 +1572,21 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 7. hide chars: replace all chars by a given char/string */
+    /* 7. convert to lower case */
+    if (strncmp (text, "lower:", 6) == 0)
+    {
+        value = eval_string_lower (text + 6);
+        goto end;
+    }
+
+    /* 8. convert to upper case */
+    if (strncmp (text, "upper:", 6) == 0)
+    {
+        value = eval_string_upper (text + 6);
+        goto end;
+    }
+
+    /* 9. hide chars: replace all chars by a given char/string */
     if (strncmp (text, "hide:", 5) == 0)
     {
         value = eval_string_hide (text + 5);
@@ -1177,7 +1594,7 @@ eval_replace_vars_cb (void *data, const char *text)
     }
 
     /*
-     * 8. cut chars:
+     * 10. cut chars:
      *   cut: max number of chars, and add an optional suffix when the
      *        string is cut
      *   cutscr: max number of chars displayed on screen, and add an optional
@@ -1194,7 +1611,7 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 9. reverse string */
+    /* 11. reverse string */
     if (strncmp (text, "rev:", 4) == 0)
     {
         value = string_reverse (text + 4);
@@ -1206,7 +1623,7 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 10. repeated string */
+    /* 12. repeated string */
     if (strncmp (text, "repeat:", 7) == 0)
     {
         value = eval_string_repeat (text + 7);
@@ -1214,7 +1631,7 @@ eval_replace_vars_cb (void *data, const char *text)
     }
 
     /*
-     * 11. length of string:
+     * 13. length of string:
      *   length: number of chars
      *   lengthscr: number of chars displayed on screen
      */
@@ -1233,35 +1650,49 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 12. regex group captured */
+    /* 14: split string */
+    if (strncmp (text, "split:", 6) == 0)
+    {
+        value = eval_string_split (text + 6);
+        goto end;
+    }
+
+    /* 15: split shell */
+    if (strncmp (text, "split_shell:", 12) == 0)
+    {
+        value = eval_string_split_shell (text + 12);
+        goto end;
+    }
+
+    /* 16. regex group captured */
     if (strncmp (text, "re:", 3) == 0)
     {
         value = eval_string_regex_group (text + 3, eval_context);
         goto end;
     }
 
-    /* 13. color code */
+    /* 17. color code */
     if (strncmp (text, "color:", 6) == 0)
     {
         value = eval_string_color (text + 6);
         goto end;
     }
 
-    /* 14. modifier */
+    /* 18. modifier */
     if (strncmp (text, "modifier:", 9) == 0)
     {
         value = eval_string_modifier (text + 9);
         goto end;
     }
 
-    /* 15. info */
+    /* 19. info */
     if (strncmp (text, "info:", 5) == 0)
     {
         value = eval_string_info (text + 5);
         goto end;
     }
 
-    /* 16. base_encode/base_decode */
+    /* 20. base_encode/base_decode */
     if (strncmp (text, "base_encode:", 12) == 0)
     {
         value = eval_string_base_encode (text + 12);
@@ -1273,14 +1704,14 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 17. current date/time */
+    /* 21. current date/time */
     if ((strncmp (text, "date", 4) == 0) && (!text[4] || (text[4] == ':')))
     {
         value = eval_string_date (text + 4);
         goto end;
     }
 
-    /* 18. environment variable */
+    /* 22. environment variable */
     if (strncmp (text, "env:", 4) == 0)
     {
         ptr_value = getenv (text + 4);
@@ -1288,7 +1719,7 @@ eval_replace_vars_cb (void *data, const char *text)
         goto end;
     }
 
-    /* 19: ternary operator: if:condition?value_if_true:value_if_false */
+    /* 23: ternary operator: if:condition?value_if_true:value_if_false */
     if (strncmp (text, "if:", 3) == 0)
     {
         value = eval_string_if (text + 3, eval_context);
@@ -1296,7 +1727,7 @@ eval_replace_vars_cb (void *data, const char *text)
     }
 
     /*
-     * 20. calculate the result of an expression
+     * 24. calculate the result of an expression
      * (with number, operators and parentheses)
      */
     if (strncmp (text, "calc:", 5) == 0)
@@ -1306,15 +1737,32 @@ eval_replace_vars_cb (void *data, const char *text)
     }
 
     /*
-     * 21. translated text
+     * 25. random number
      */
-    if (strncmp (text, "translate:", 10) == 0)
+    if (strncmp (text, "random:", 7) == 0)
     {
-        value = eval_translate (text + 10);
+        value = eval_string_random (text + 7);
         goto end;
     }
 
-    /* 22. option: if found, return this value */
+    /*
+     * 26. translated text
+     */
+    if (strncmp (text, "translate:", 10) == 0)
+    {
+        value = eval_string_translate (text + 10);
+        goto end;
+    }
+
+    /* 27. define a variable */
+    if (strncmp (text, "define:", 7) == 0)
+    {
+        eval_string_define (text + 7, eval_context);
+        value = strdup ("");
+        goto end;
+    }
+
+    /* 28. option: if found, return this value */
     if (strncmp (text, "sec.data.", 9) == 0)
     {
         ptr_value = hashtable_get (secure_hashtable_data, text + 9);
@@ -1357,7 +1805,7 @@ eval_replace_vars_cb (void *data, const char *text)
         }
     }
 
-    /* 23. local variable in buffer */
+    /* 29. local variable in buffer */
     ptr_buffer = hashtable_get (eval_context->pointers, "buffer");
     if (ptr_buffer)
     {
@@ -1369,7 +1817,7 @@ eval_replace_vars_cb (void *data, const char *text)
         }
     }
 
-    /* 24. hdata */
+    /* 30. hdata */
     value = eval_string_hdata (text, eval_context);
 
 end:
@@ -1839,6 +2287,7 @@ eval_replace_regex (const char *string, regex_t *regex, const char *replace,
     snprintf (result, length, "%s", string);
 
     eval_context->regex = &eval_regex;
+    eval_context->regex_replacement_index = 1;
 
     start_offset = 0;
 
@@ -1922,6 +2371,8 @@ eval_replace_regex (const char *string, regex_t *regex, const char *replace,
 
         if (!result[start_offset])
             break;
+
+        (eval_context->regex_replacement_index)++;
     }
 
 end:
@@ -1976,6 +2427,7 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
                  struct t_hashtable *extra_vars, struct t_hashtable *options)
 {
     struct t_eval_context context, *eval_context;
+    struct t_hashtable *user_vars;
     int condition, rc, pointers_allocated, regex_allocated, debug_id;
     int ptr_window_added, ptr_buffer_added;
     long number;
@@ -1990,6 +2442,7 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
         return NULL;
 
     condition = 0;
+    user_vars = NULL;
     pointers_allocated = 0;
     regex_allocated = 0;
     regex = NULL;
@@ -2014,14 +2467,24 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
         pointers_allocated = 1;
     }
 
+    user_vars = hashtable_new (32,
+                               WEECHAT_HASHTABLE_STRING,
+                               WEECHAT_HASHTABLE_STRING,
+                               NULL,
+                               NULL);
+
     eval_context = &context;
 
     eval_context->pointers = pointers;
     eval_context->extra_vars = extra_vars;
+    eval_context->user_vars = user_vars;
     eval_context->extra_vars_eval = 0;
     eval_context->prefix = default_prefix;
+    eval_context->length_prefix = strlen (eval_context->prefix);
     eval_context->suffix = default_suffix;
+    eval_context->length_suffix = strlen (eval_context->suffix);
     eval_context->regex = NULL;
+    eval_context->regex_replacement_index = 1;
     eval_context->recursion_count = 0;
     eval_context->debug_level = 0;
     eval_context->debug_depth = 0;
@@ -2066,12 +2529,18 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
         /* check for custom prefix */
         ptr_value = hashtable_get (options, "prefix");
         if (ptr_value && ptr_value[0])
+        {
             eval_context->prefix = ptr_value;
+            eval_context->length_prefix = strlen (eval_context->prefix);
+        }
 
         /* check for custom suffix */
         ptr_value = hashtable_get (options, "suffix");
         if (ptr_value && ptr_value[0])
+        {
             eval_context->suffix = ptr_value;
+            eval_context->length_suffix = strlen (eval_context->suffix);
+        }
 
         /* check for regex */
         ptr_value = hashtable_get (options, "regex");
@@ -2148,6 +2617,8 @@ eval_expression (const char *expr, struct t_hashtable *pointers,
         if (ptr_buffer_added)
             hashtable_remove (pointers, "buffer");
     }
+    if (user_vars)
+        hashtable_free (user_vars);
     if (regex && regex_allocated)
     {
         regfree (regex);
