@@ -1,7 +1,7 @@
 /*
  * trigger.c - trigger plugin for WeeChat
  *
- * Copyright (C) 2014-2016 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2022 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -38,37 +38,41 @@ WEECHAT_PLUGIN_DESCRIPTION(N_("Text replacement and command execution on events
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(10000);
+WEECHAT_PLUGIN_PRIORITY(13000);
 
 struct t_weechat_plugin *weechat_trigger_plugin = NULL;
 
 char *trigger_option_string[TRIGGER_NUM_OPTIONS] =
 { "enabled", "hook", "arguments", "conditions", "regex", "command",
-  "return_code" };
+  "return_code", "post_action" };
 char *trigger_option_default[TRIGGER_NUM_OPTIONS] =
-{ "on", "signal", "", "", "", "", "ok" };
+{ "on", "signal", "", "", "", "", "ok", "none" };
 
 char *trigger_hook_type_string[TRIGGER_NUM_HOOK_TYPES] =
-{ "signal", "hsignal", "modifier", "print", "command", "command_run", "timer",
-  "config", "focus" };
+{ "signal", "hsignal", "modifier", "line", "print", "command", "command_run",
+  "timer", "config", "focus", "info", "info_hashtable" };
 char *trigger_hook_option_values =
-    "signal|hsignal|modifier|print|command|command_run|timer|config|focus";
+    "signal|hsignal|modifier|line|print|command|command_run|timer|config|"
+    "focus|info|info_hashtable";
 char *trigger_hook_default_arguments[TRIGGER_NUM_HOOK_TYPES] =
-{ "xxx", "xxx", "xxx", "", "cmd;desc;args;args_desc;%(buffers_names)", "/cmd",
-  "60000;0;0", "xxx", "chat" };
+{ "xxx", "xxx", "xxx", "", "", "cmd;desc;args;args_desc;%(buffers_names)",
+  "/cmd", "60000;0;0", "xxx", "chat", "xxx", "xxx" };
 char *trigger_hook_default_rc[TRIGGER_NUM_HOOK_TYPES] =
-{ "ok,ok_eat,error", "ok,ok_eat,error", "", "ok,error", "ok,error",
-  "ok,ok_eat,error", "ok", "ok", "" };
+{ "ok,ok_eat,error", "ok,ok_eat,error", "", "", "ok,error", "ok,error",
+  "ok,ok_eat,error", "ok", "ok", "", "", "" };
 
 char *trigger_hook_regex_default_var[TRIGGER_NUM_HOOK_TYPES] =
-{ "tg_signal_data", "", "tg_string", "tg_message", "tg_argv_eol1", "tg_command",
-  "tg_remaining_calls", "tg_value", "" };
+{ "tg_signal_data", "", "tg_string", "message", "tg_message", "tg_argv_eol1",
+  "tg_command", "tg_remaining_calls", "tg_value", "", "tg_info", "" };
 
 char *trigger_return_code_string[TRIGGER_NUM_RETURN_CODES] =
 { "ok", "ok_eat", "error" };
 int trigger_return_code[TRIGGER_NUM_RETURN_CODES] =
 { WEECHAT_RC_OK, WEECHAT_RC_OK_EAT, WEECHAT_RC_ERROR };
 
+char *trigger_post_action_string[TRIGGER_NUM_POST_ACTIONS] =
+{ "none", "disable", "delete" };
+
 struct t_trigger *triggers = NULL;          /* first trigger                */
 struct t_trigger *last_trigger = NULL;      /* last trigger                 */
 int triggers_count = 0;                     /* number of triggers           */
@@ -145,6 +149,27 @@ trigger_search_return_code (const char *return_code)
     return -1;
 }
 
+/*
+ * Searches for trigger post action.
+ *
+ * Returns index of post action in enum t_trigger_post_action, -1 if not found.
+ */
+
+int
+trigger_search_post_action (const char *post_action)
+{
+    int i;
+
+    for (i = 0; i < TRIGGER_NUM_POST_ACTIONS; i++)
+    {
+        if (weechat_strcasecmp (trigger_post_action_string[i], post_action) == 0)
+            return i;
+    }
+
+    /* post action not found */
+    return -1;
+}
+
 /*
  * Searches for a trigger by name.
  *
@@ -182,6 +207,10 @@ trigger_search_with_option (struct t_config_option *option)
     const char *ptr_name;
     char *pos_option;
     struct t_trigger *ptr_trigger;
+    int num_chars;
+
+    if (!option)
+        return NULL;
 
     ptr_name = weechat_hdata_string (weechat_hdata_get ("config_option"),
                                      option, "name");
@@ -192,10 +221,12 @@ trigger_search_with_option (struct t_config_option *option)
     if (!pos_option)
         return NULL;
 
+    num_chars = weechat_utf8_pos (ptr_name, pos_option - ptr_name);
+
     for (ptr_trigger = triggers; ptr_trigger;
          ptr_trigger = ptr_trigger->next_trigger)
     {
-        if (weechat_strncasecmp (ptr_trigger->name, ptr_name, pos_option - ptr_name) == 0)
+        if (weechat_strncasecmp (ptr_trigger->name, ptr_name, num_chars) == 0)
             break;
     }
 
@@ -238,53 +269,60 @@ trigger_unhook (struct t_trigger *trigger)
 void
 trigger_hook (struct t_trigger *trigger)
 {
-    char **argv, **argv_eol, *tags, *message, *error1, *error2, *error3;
+    char **argv, **argv_eol, *buffer_type, *buffer_name, *tags, *message;
+    char *error1, *error2, *error3;
+    char *eval_desc, *eval_args, *eval_desc_args, *eval_completion;
     int i, argc, strip_colors;
     long interval, align_second, max_calls;
+    struct t_hashtable *extra_vars;
 
     if (!weechat_config_boolean (trigger->options[TRIGGER_OPTION_ENABLED]))
         return;
 
     trigger_unhook (trigger);
 
-    argv = weechat_string_split (weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
-                                 ";", 0, 0, &argc);
-    argv_eol = weechat_string_split (weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
-                                     ";", 1, 0, NULL);
+    argv = weechat_string_split (
+        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+        ";",
+        NULL,
+        0,
+        0,
+        &argc);
+    argv_eol = weechat_string_split (
+        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+        ";",
+        NULL,
+        WEECHAT_STRING_SPLIT_KEEP_EOL,
+        0,
+        NULL);
 
     switch (weechat_config_integer (trigger->options[TRIGGER_OPTION_HOOK]))
     {
         case TRIGGER_HOOK_SIGNAL:
             if (argv && (argc >= 1))
             {
-                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                trigger->hooks = malloc (sizeof (trigger->hooks[0]));
                 if (trigger->hooks)
                 {
-                    trigger->hooks_count = argc;
-                    for (i = 0; i < argc; i++)
-                    {
-                        trigger->hooks[i] = weechat_hook_signal (
-                            argv[i],
-                            &trigger_callback_signal_cb,
-                            trigger);
-                    }
+                    trigger->hooks_count = 1;
+                    trigger->hooks[0] = weechat_hook_signal (
+                        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+                        &trigger_callback_signal_cb,
+                        trigger, NULL);
                 }
             }
             break;
         case TRIGGER_HOOK_HSIGNAL:
             if (argv && (argc >= 1))
             {
-                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                trigger->hooks = malloc (sizeof (trigger->hooks[0]));
                 if (trigger->hooks)
                 {
-                    trigger->hooks_count = argc;
-                    for (i = 0; i < argc; i++)
-                    {
-                        trigger->hooks[i] = weechat_hook_hsignal (
-                            argv[i],
-                            &trigger_callback_hsignal_cb,
-                            trigger);
-                    }
+                    trigger->hooks_count = 1;
+                    trigger->hooks[0] = weechat_hook_hsignal (
+                        weechat_config_string (trigger->options[TRIGGER_OPTION_ARGUMENTS]),
+                        &trigger_callback_hsignal_cb,
+                        trigger, NULL);
                 }
             }
             break;
@@ -300,11 +338,35 @@ trigger_hook (struct t_trigger *trigger)
                         trigger->hooks[i] = weechat_hook_modifier (
                             argv[i],
                             &trigger_callback_modifier_cb,
-                            trigger);
+                            trigger, NULL);
                     }
                 }
             }
             break;
+        case TRIGGER_HOOK_LINE:
+            buffer_type = NULL;
+            buffer_name = NULL;
+            tags = NULL;
+            if (argv && (argc >= 1))
+            {
+                buffer_type = argv[0];
+                if ((argc >= 2) && (strcmp (argv[1], "*") != 0))
+                    buffer_name = argv[1];
+                if ((argc >= 3) && (strcmp (argv[2], "*") != 0))
+                    tags = argv[2];
+            }
+            trigger->hooks = malloc (sizeof (trigger->hooks[0]));
+            if (trigger->hooks)
+            {
+                trigger->hooks_count = 1;
+                trigger->hooks[0] = weechat_hook_line (
+                    buffer_type,
+                    buffer_name,
+                    tags,
+                    &trigger_callback_line_cb,
+                    trigger, NULL);
+            }
+            break;
         case TRIGGER_HOOK_PRINT:
             tags = NULL;
             message = NULL;
@@ -330,7 +392,7 @@ trigger_hook (struct t_trigger *trigger)
                     message,
                     strip_colors,
                     &trigger_callback_print_cb,
-                    trigger);
+                    trigger, NULL);
             }
             break;
         case TRIGGER_HOOK_COMMAND:
@@ -339,15 +401,42 @@ trigger_hook (struct t_trigger *trigger)
                 trigger->hooks = malloc (sizeof (trigger->hooks[0]));
                 if (trigger->hooks)
                 {
+                    extra_vars = weechat_hashtable_new (32,
+                                                        WEECHAT_HASHTABLE_STRING,
+                                                        WEECHAT_HASHTABLE_STRING,
+                                                        NULL, NULL);
+                    if (extra_vars)
+                    {
+                        weechat_hashtable_set (extra_vars, "tg_trigger_name",
+                                               trigger->name);
+                    }
                     trigger->hooks_count = 1;
+                    eval_desc = (argc > 1) ? weechat_string_eval_expression (
+                        argv[1], NULL, extra_vars, NULL) : NULL;
+                    eval_args = (argc > 2) ? weechat_string_eval_expression (
+                        argv[2], NULL, extra_vars, NULL) : NULL;
+                    eval_desc_args = (argc > 3) ? weechat_string_eval_expression (
+                        argv[3], NULL, extra_vars, NULL) : NULL;
+                    eval_completion = (argc > 4) ? weechat_string_eval_expression (
+                        argv[4], NULL, extra_vars, NULL) : NULL;
                     trigger->hooks[0] = weechat_hook_command (
                         argv[0],  /* command */
-                        (argc > 1) ? argv[1] : "",  /* description */
-                        (argc > 2) ? argv[2] : "",  /* arguments */
-                        (argc > 3) ? argv[3] : "",  /* description of args */
-                        (argc > 4) ? argv[4] : "",  /* completion */
+                        (eval_desc) ? eval_desc : "",
+                        (eval_args) ? eval_args : "",
+                        (eval_desc_args) ? eval_desc_args : "",
+                        (eval_completion) ? eval_completion : "",
                         &trigger_callback_command_cb,
-                        trigger);
+                        trigger, NULL);
+                    if (eval_desc)
+                        free (eval_desc);
+                    if (eval_args)
+                        free (eval_args);
+                    if (eval_desc_args)
+                        free (eval_desc_args);
+                    if (eval_completion)
+                        free (eval_completion);
+                    if (extra_vars)
+                        weechat_hashtable_free (extra_vars);
                 }
             }
             break;
@@ -363,7 +452,7 @@ trigger_hook (struct t_trigger *trigger)
                         trigger->hooks[i] = weechat_hook_command_run (
                             argv[i],
                             &trigger_callback_command_run_cb,
-                            trigger);
+                            trigger, NULL);
                     }
                 }
             }
@@ -393,7 +482,7 @@ trigger_hook (struct t_trigger *trigger)
                             (int)align_second,
                             (int)max_calls,
                             &trigger_callback_timer_cb,
-                            trigger);
+                            trigger, NULL);
                     }
                 }
             }
@@ -410,7 +499,7 @@ trigger_hook (struct t_trigger *trigger)
                         trigger->hooks[i] = weechat_hook_config (
                             argv[i],
                             &trigger_callback_config_cb,
-                            trigger);
+                            trigger, NULL);
                     }
                 }
             }
@@ -427,7 +516,46 @@ trigger_hook (struct t_trigger *trigger)
                         trigger->hooks[i] = weechat_hook_focus (
                             argv[i],
                             &trigger_callback_focus_cb,
-                            trigger);
+                            trigger, NULL);
+                    }
+                }
+            }
+            break;
+        case TRIGGER_HOOK_INFO:
+            if (argv && (argc >= 1))
+            {
+                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                if (trigger->hooks)
+                {
+                    trigger->hooks_count = argc;
+                    for (i = 0; i < argc; i++)
+                    {
+                        trigger->hooks[i] = weechat_hook_info (
+                            argv[i],
+                            NULL,
+                            NULL,
+                            &trigger_callback_info_cb,
+                            trigger, NULL);
+                    }
+                }
+            }
+            break;
+        case TRIGGER_HOOK_INFO_HASHTABLE:
+            if (argv && (argc >= 1))
+            {
+                trigger->hooks = malloc (argc * sizeof (trigger->hooks[0]));
+                if (trigger->hooks)
+                {
+                    trigger->hooks_count = argc;
+                    for (i = 0; i < argc; i++)
+                    {
+                        trigger->hooks[i] = weechat_hook_info_hashtable (
+                            argv[i],
+                            NULL,
+                            NULL,
+                            NULL,
+                            &trigger_callback_info_hashtable_cb,
+                            trigger, NULL);
                     }
                 }
             }
@@ -458,6 +586,9 @@ trigger_regex_free (int *regex_count, struct t_trigger_regex **regex)
 {
     int i;
 
+    if (!regex_count || !regex)
+        return;
+
     if (*regex_count > 0)
     {
         for (i = 0; i < *regex_count; i++)
@@ -544,8 +675,6 @@ trigger_regex_split (const char *str_regex,
         length_delimiter = strlen (delimiter);
 
         ptr_regex = pos;
-        if (!ptr_regex[0])
-            goto format_error;
 
         /* search the start of replacement string */
         pos_replace = strstr (ptr_regex, delimiter);
@@ -695,8 +824,9 @@ trigger_split_command (const char *command,
 }
 
 /*
- * Checks if a trigger name is valid: it must not start with "-" and not have
- * any spaces.
+ * Checks if a trigger name is valid:
+ *   - it must not start with "-"
+ *   - it must not have any space/period.
  *
  * Returns:
  *   1: name is valid
@@ -812,10 +942,10 @@ trigger_add (struct t_trigger *trigger,
         /* add trigger to end of list */
         trigger->prev_trigger = *last_list_trigger;
         trigger->next_trigger = NULL;
-        if (!*list_triggers)
-            *list_triggers = trigger;
-        else
+        if (*last_list_trigger)
             (*last_list_trigger)->next_trigger = trigger;
+        else
+            *list_triggers = trigger;
         *last_list_trigger = trigger;
     }
 }
@@ -872,7 +1002,8 @@ trigger_new_with_options (const char *name, struct t_config_option **options)
 struct t_trigger *
 trigger_new (const char *name, const char *enabled, const char *hook,
              const char *arguments, const char *conditions, const char *regex,
-             const char *command, const char *return_code)
+             const char *command, const char *return_code,
+             const char *post_action)
 {
     struct t_config_option *option[TRIGGER_NUM_OPTIONS];
     const char *value[TRIGGER_NUM_OPTIONS];
@@ -890,6 +1021,13 @@ trigger_new (const char *name, const char *enabled, const char *hook,
         return NULL;
     }
 
+    /* look for post action */
+    if (post_action && post_action[0]
+        && (trigger_search_post_action (post_action) < 0))
+    {
+        return NULL;
+    }
+
     value[TRIGGER_OPTION_ENABLED] = enabled;
     value[TRIGGER_OPTION_HOOK] = hook;
     value[TRIGGER_OPTION_ARGUMENTS] = arguments;
@@ -897,6 +1035,7 @@ trigger_new (const char *name, const char *enabled, const char *hook,
     value[TRIGGER_OPTION_REGEX] = regex;
     value[TRIGGER_OPTION_COMMAND] = command;
     value[TRIGGER_OPTION_RETURN_CODE] = return_code;
+    value[TRIGGER_OPTION_POST_ACTION] = post_action;
 
     for (i = 0; i < TRIGGER_NUM_OPTIONS; i++)
     {
@@ -933,7 +1072,8 @@ trigger_create_default ()
                      trigger_config_default_list[i][4],   /* conditions */
                      trigger_config_default_list[i][5],   /* regex */
                      trigger_config_default_list[i][6],   /* command */
-                     trigger_config_default_list[i][7]);  /* return code */
+                     trigger_config_default_list[i][7],   /* return code */
+                     trigger_config_default_list[i][8]);  /* post action */
     }
 }
 
@@ -951,7 +1091,7 @@ trigger_rename (struct t_trigger *trigger, const char *name)
     int length, i;
     char *option_name;
 
-    if (!name || !name[0] || !trigger_name_valid (name)
+    if (!trigger || !name || !name[0] || !trigger_name_valid (name)
         || trigger_search (name))
     {
         return 0;
@@ -1003,7 +1143,7 @@ trigger_rename (struct t_trigger *trigger, const char *name)
 struct t_trigger *
 trigger_copy (struct t_trigger *trigger, const char *name)
 {
-    if (!name || !name[0] || !trigger_name_valid (name)
+    if (!trigger || !name || !name[0] || !trigger_name_valid (name)
         || trigger_search (name))
     {
         return NULL;
@@ -1017,7 +1157,8 @@ trigger_copy (struct t_trigger *trigger, const char *name)
         weechat_config_string (trigger->options[TRIGGER_OPTION_CONDITIONS]),
         weechat_config_string (trigger->options[TRIGGER_OPTION_REGEX]),
         weechat_config_string (trigger->options[TRIGGER_OPTION_COMMAND]),
-        weechat_config_string (trigger->options[TRIGGER_OPTION_RETURN_CODE]));
+        weechat_config_string (trigger->options[TRIGGER_OPTION_RETURN_CODE]),
+        weechat_config_string (trigger->options[TRIGGER_OPTION_POST_ACTION]));
 }
 
 /*
@@ -1105,6 +1246,9 @@ trigger_print_log ()
         weechat_log_printf ("  return_code . . . . . . : %d ('%s')",
                             weechat_config_integer (ptr_trigger->options[TRIGGER_OPTION_RETURN_CODE]),
                             trigger_return_code_string[weechat_config_integer (ptr_trigger->options[TRIGGER_OPTION_RETURN_CODE])]);
+        weechat_log_printf ("  post_action . . . . . . : %d ('%s')",
+                            weechat_config_integer (ptr_trigger->options[TRIGGER_OPTION_POST_ACTION]),
+                            trigger_post_action_string[weechat_config_integer (ptr_trigger->options[TRIGGER_OPTION_POST_ACTION])]);
         weechat_log_printf ("  hooks_count . . . . . . : %d",    ptr_trigger->hooks_count);
         weechat_log_printf ("  hooks . . . . . . . . . : 0x%lx", ptr_trigger->hooks);
         for (i = 0; i < ptr_trigger->hooks_count; i++)
@@ -1151,10 +1295,12 @@ trigger_print_log ()
  */
 
 int
-trigger_debug_dump_cb (void *data, const char *signal, const char *type_data,
+trigger_debug_dump_cb (const void *pointer, void *data,
+                       const char *signal, const char *type_data,
                        void *signal_data)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
     (void) type_data;
@@ -1183,7 +1329,9 @@ trigger_debug_dump_cb (void *data, const char *signal, const char *type_data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    int i, upgrading;
+    /* make C compiler happy */
+    (void) argc;
+    (void) argv;
 
     weechat_plugin = plugin;
 
@@ -1197,22 +1345,12 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     trigger_config_read ();
 
     /* hook some signals */
-    weechat_hook_signal ("debug_dump", &trigger_debug_dump_cb, NULL);
+    weechat_hook_signal ("debug_dump", &trigger_debug_dump_cb, NULL, NULL);
 
     /* hook completions */
     trigger_completion_init ();
 
-    /* look at arguments */
-    upgrading = 0;
-    for (i = 0; i < argc; i++)
-    {
-        if (weechat_strcasecmp (argv[i], "--upgrade") == 0)
-        {
-            upgrading = 1;
-        }
-    }
-
-    if (upgrading)
+    if (weechat_trigger_plugin->upgrading)
         trigger_buffer_set_callbacks ();
 
     return WEECHAT_RC_OK;
